@@ -132,7 +132,7 @@ void MainWindow::init()
 void MainWindow::closeEvent(QCloseEvent* closeEvent)
 {
   // Logout first?
-  ProviderInterfacePtr provider = d->messageWidget->provider();
+  ProviderInterface* provider = d->messageWidget->provider();
 
   if ( provider && provider->isLoggedIn() )
   {
@@ -147,8 +147,8 @@ void MainWindow::closeEvent(QCloseEvent* closeEvent)
     progressDialog->show();
 
     // Close the information dialog and this window
-    connect(provider.data(), SIGNAL(logoutReply(bool)), progressDialog, SLOT(accept()));
-    connect(provider.data(), SIGNAL(logoutReply(bool)), this, SLOT(close()));
+    connect(provider, SIGNAL(logoutReply(bool)), progressDialog, SLOT(accept()));
+    connect(provider, SIGNAL(logoutReply(bool)), this, SLOT(close()));
 
     // Log out now
     provider->logout();
