@@ -29,7 +29,13 @@
 // Global includes
 #include <QtGui/QProgressDialog>
 #include <QtGui/QMessageBox>
-#include <QtMaemo5/QMaemo5ListPickSelector>
+
+#ifdef Q_WS_MAEMO_5
+# include <QtMaemo5/QMaemo5ValueButton>
+# include <QtMaemo5/QMaemo5ListPickSelector>
+#else // Q_WS_MAEMO_5
+# include <QtGui/QComboBox>
+#endif // Q_WS_MAEMO_5
 
 //////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////
@@ -48,10 +54,16 @@ public:
   // Properties
   Ui_MessageWidget ui;
 
-  ProviderInterfacePtr provider;
+#ifdef Q_WS_MAEMO_5
+  QMaemo5ValueButton* chooseProviderButton;
+  QMaemo5ListPickSelector selector;
+#else // Q_WS_MAEMO_5
+  QComboBox* chooseProviderComboBox;
+#endif // Q_WS_MAEMO_5
+
+  QPointer<ProviderInterface> provider;
 
   AccountsModel accountsModel;
-  QMaemo5ListPickSelector selector;
 
   // Functions
   void init();
@@ -81,12 +93,27 @@ void MessageWidgetPrivate::init()
   connect(ui.chooseContactButton, SIGNAL(clicked()), this, SLOT(onChooseContactButtonClicked()));
   connect(ui.sendButton         , SIGNAL(clicked()), this, SLOT(onSendButtonClicked()));
 
+#ifdef Q_WS_MAEMO_5
+  // Setup the value button
+  chooseProviderButton = new QMaemo5ValueButton(tr("Provider"), self);
+  ui.providerPlaceholderLayout->addWidget(chooseProviderButton);
+
   // Setup the pick selector
   connect(&selector, SIGNAL(selected(QString)), this, SLOT(onProviderSelectionChanged()));
   selector.setModel(&accountsModel);
 
   // Set the pick selector
-  ui.chooseProviderButton->setPickSelector(&selector);
+  chooseProviderButton->setPickSelector(&selector);
+#else // Q_WS_MAEMO_5
+  // Setup the combobox
+  chooseProviderComboBox = new QComboBox(self);
+  ui.providerPlaceholderLayout->addWidget(chooseProviderComboBox);
+
+  // Setup the seleciton changes
+  connect(chooseProviderComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onProviderSelectionChanged()));
+  chooseProviderComboBox->setModel(&accountsModel);
+  chooseProviderComboBox->setCurrentIndex(0);
+#endif // Q_WS_MAEMO_5
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -207,8 +234,14 @@ void MessageWidgetPrivate::onSendButtonClicked()
 
 void MessageWidgetPrivate::onProviderSelectionChanged()
 {
+#ifdef Q_WS_MAEMO_5
+  int index = selector.currentIndex();
+#else // Q_WS_MAEMO_5
+  int index = chooseProviderComboBox->currentIndex();
+#endif // Q_WS_MAEMO_5
+
   // Get the current index
-  QModelIndex currentIndex = accountsModel.index(selector.currentIndex(), selector.modelColumn());
+  QModelIndex currentIndex = accountsModel.index(index, 0);
   if ( !currentIndex.isValid() )
     return;
 
@@ -218,7 +251,9 @@ void MessageWidgetPrivate::onProviderSelectionChanged()
     return;
 
   // Give the button a shiny icon
-  ui.chooseProviderButton->setIcon( qvariant_cast<QIcon>(currentIndex.data(Qt::DecorationRole)) );
+#ifdef Q_WS_MAEMO_5
+  chooseProviderButton->setIcon( qvariant_cast<QIcon>(currentIndex.data(Qt::DecorationRole)) );
+#endif // Q_WS_MAEMO_5
 
   // Change the current provider
   AccountsPtr accounts = accountsModel.accounts();
@@ -247,7 +282,7 @@ MessageWidget::~MessageWidget()
 
 //////////////////////////////////////////////////////////////////////////
 
-void MessageWidget::setProvider(const ProviderInterfacePtr& provider)
+void MessageWidget::setProvider(ProviderInterface* provider)
 {
   // Something changed?
   if ( d->provider == provider )
@@ -256,10 +291,10 @@ void MessageWidget::setProvider(const ProviderInterfacePtr& provider)
   // Disconnect the old provider
   if ( d->provider )
   {
-    disconnect(d->provider.data(), SIGNAL(error(QString)), d, SLOT(onError(QString)));
-    disconnect(d->provider.data(), SIGNAL(loginReply(bool)), d, SLOT(onLoginReply(bool)));
-    disconnect(d->provider.data(), SIGNAL(logoutReply(bool)), d, SLOT(onLogoutReply(bool)));
-    disconnect(d->provider.data(), SIGNAL(freeSmsCountChanged(int)), d, SLOT(onFreeSmsCountChanged(int)));
+    disconnect(d->provider, SIGNAL(error(QString)), d, SLOT(onError(QString)));
+    disconnect(d->provider, SIGNAL(loginReply(bool)), d, SLOT(onLoginReply(bool)));
+    disconnect(d->provider, SIGNAL(logoutReply(bool)), d, SLOT(onLogoutReply(bool)));
+    disconnect(d->provider, SIGNAL(freeSmsCountChanged(int)), d, SLOT(onFreeSmsCountChanged(int)));
 
     // Logout
     if ( d->provider->isLoggedIn() )
@@ -275,29 +310,32 @@ void MessageWidget::setProvider(const ProviderInterfacePtr& provider)
     d->onFreeSmsCountChanged(-1);
 
     // Connect signals
-    connect(d->provider.data(), SIGNAL(error(QString)), d, SLOT(onError(QString)));
-    connect(d->provider.data(), SIGNAL(loginReply(bool)), d, SLOT(onLoginReply(bool)));
-    connect(d->provider.data(), SIGNAL(logoutReply(bool)), d, SLOT(onLogoutReply(bool)));
-    connect(d->provider.data(), SIGNAL(freeSmsCountChanged(int)), d, SLOT(onFreeSmsCountChanged(int)));
+    connect(d->provider, SIGNAL(error(QString)), d, SLOT(onError(QString)));
+    connect(d->provider, SIGNAL(loginReply(bool)), d, SLOT(onLoginReply(bool)));
+    connect(d->provider, SIGNAL(logoutReply(bool)), d, SLOT(onLogoutReply(bool)));
+    connect(d->provider, SIGNAL(freeSmsCountChanged(int)), d, SLOT(onFreeSmsCountChanged(int)));
 
     // Show a dialog that we're logging out first
-    QProgressDialog* progressDialog = new QProgressDialog(this);
-    progressDialog->setAttribute(Qt::WA_DeleteOnClose);
-    progressDialog->setWindowTitle( tr("Logging in ...") );
-    progressDialog->setRange(0, 0);
-    progressDialog->show();
+    if ( !d->provider->isLoggedIn() )
+    {
+      QProgressDialog* progressDialog = new QProgressDialog(this);
+      progressDialog->setAttribute(Qt::WA_DeleteOnClose);
+      progressDialog->setWindowTitle( tr("Logging in ...") );
+      progressDialog->setRange(0, 0);
+      progressDialog->show();
 
-    // Close the information dialog and this window
-    connect(provider.data(), SIGNAL(loginReply(bool)), progressDialog, SLOT(accept()));
+      // Close the information dialog and this window
+      connect(provider, SIGNAL(loginReply(bool)), progressDialog, SLOT(accept()));
 
-    // Login
-    d->provider->login();
+      // Login
+      d->provider->login();
+    }
   }
 }
 
 //////////////////////////////////////////////////////////////////////////
 
-ProviderInterfacePtr MessageWidget::provider() const
+ProviderInterface* MessageWidget::provider() const
 {
   return d->provider;
 }
@@ -307,7 +345,12 @@ ProviderInterfacePtr MessageWidget::provider() const
 void MessageWidget::setAccounts(const AccountsPtr& accounts)
 {
   d->accountsModel.setAccounts(accounts);
+
+#ifdef Q_WS_MAEMO_5
   d->selector.setCurrentIndex(0);
+#else // Q_WS_MAEMO_5
+  d->chooseProviderComboBox->setCurrentIndex(0);
+#endif // Q_WS_MAEMO_5
 }
 
 //////////////////////////////////////////////////////////////////////////
